; Kopterlifter v1.0 by Michael Vincent (michael@detachedsolutions.com)
;
; You may modify the source code for personal purposes but do not distribute
; altered versions of this program.
;
.nolist
#include "ti83plus.inc"
#include "mirage.inc"
;-----> Ion Functions
lVectors	=cmdshadow+80
ionVersion	=lVectors
ionRandom	=lVectors+3
ionPutSprite	=lVectors+6
ionLargeSprite	=lVectors+9
ionGetPixel	=lVectors+12
ionFastCopy	=lVectors+15
ionDetect	=lVectors+18
ionDecompress	=lVectors+21
.list
#define ground_height 8
#define objects savesscreen	;objects in the game (helicopter, buildings, et cetera)
#define bullets savesscreen+objects_end-objects_initial	;bullets (missiles too)
#define people bullets+(10*max_bullets)
#define gravconstant (0<<6) | 29	;gravity displacement per frame
#define sidewaysv (1<<6) | 0		;sideways motion per frame
#define x_acceleration $03		;X acceleration from left/right keys
#define y_acceleration $01		;Y acceleration from up/down keys
#define house0 90	;X coordinate of first house
#define house1 250
#define house2 410
#define house3 570
#define max_bullets 16		;maximum number of bullets/missiles allowed
#define reload_time 30		;time allowed between shots
#define hostage_time 20		;time in between hostage releases
#define people_update 15	;interval for people running, waving, et cetera
#define people_velocity $10	;speed of a running hostage
#define t_running 75		;length of time a hostage runs for
#define invisible_wall 750	;wall between houses and base for hostages to run into
#define vision_radius 110<<6	;how close the chopper must be to a hostage before he moves
#define tank_velocity $4A	;how fast a tank moves
#define tank_reload 30		;how long a tank waits in between shots
#define tank_gravity 3		;gravity for tank-fired bullets
#define blimp_speed 1<<6	;velocity of an evil blimp
#define blimp_yspeed $14	;y tracking of the blimp
#define blimp_missilev 1<<6 | $05	;initial x velocity of blimp missiles
#define blimp_missilea $01		;x velocity deceleration for blimp missiles
#define blimp_reloadtime 30	;reload time for blimp to fire missile again
#define blimp_missilem 54	;the number of Y movements a missile has
#define blimp_dead 2		;time an exploded blimp remains before vanishing

#define counter appbackupscreen	;one byte, interrupt counter
#define window_x counter+1
#define heliangle window_x+2	;one byte, values are 0-2 where 0 is level
#define heliface heliangle+1	;one byte, 0 for center, 1 for right, 2 for left
#define heliblade heliface+1	;one byte, the status of the blade
#define curkey heliblade+1	;one byte, the current key
#define lastkey curkey+1	;one byte, the previous key
#define chop_vert lastkey+1	;two bytes, the helicopter's vertical velocity
#define chop_horz chop_vert+2	;two bytes, the helicopter's horizontal velocity
#define firetime chop_horz+2	;one byte, the length of time before the user can fire the gun
#define collision_height firetime+1	;one byte - temporary space for collision checking
#define collision_width collision_height+1	;one byte - ditto
#define people_active collision_width+1		;byte, the number of hostages out and about
#define people_chopper people_active+1		;byte, number of people board chopper
#define people_rescued people_chopper+1		;byte, number of people safely at base
#define sortie people_rescued+1			;byte (0-2)...number of current sortie
#define base_run sortie+1		;byte...time since last hostage ran into the base
#define blimp_state base_run+1		;byte...blimp state. 0 = inactive. 1 = crashing. 2 = moving left. 3 = moving right
#define blimp_byte blimp_state+1	;byte of random junk...used to slow down missiles
#define people_killed blimp_byte+1	;byte...number of people killed

#define Sprite_Helicopter_Left_s0 tempswaparea
#define Sprite_Helicopter_Left_s1 tempswaparea+48
#define Sprite_Helicopter_Left_s2 tempswaparea+96
.org $9D93
	.db $BB,$6D
	ret
	jr nc,start_me
	.db "Kopterlifter",0
start_me:
	im 1
	bcall(_clrlcdfull)
	bcall(_homeup)
	ld hl,title_text
	bcall(_puts)
	ld b,7
start_loop2:
	push bc
	call vputsp
	pop bc
	djnz start_loop2
start_loop1:
	ei \ halt
	bcall(_getcsc)
	or a
	jr z,start_loop1
start:
	set indiconly,(iy+indicflags)
	;Initialize objects
	ld hl,objects_initial
	ld de,objects
	ld bc,objects_end-objects_initial
	ldir
	ld hl,people_data
	ld de,people
	ld bc,64*5
	ldir
	;Bullet format:
	;.db 0 - bit 7 set for active bullet
	;^- Lower nibble is bullet type - 1 for helicopter bullet, 2 for tank bullet (has gravity), 4 for blimp missile
	;.dw x position, y position, x velocity, y velocity
	;.db 0 <-- needed status flags?
	;Create left helicopter sprite as a mirror of the right one
	ld de,Sprite_Helicopter_Right_a0s0
	ld hl,tempswaparea
	ld b,8*2*3*3
Mirror_Loop:
	ld a,(de)
	push bc
	ld b,8
Mirror_Loop_2:
	add a,a
	rr (hl)
	djnz Mirror_Loop_2
	pop bc
	inc hl
	inc de
	djnz Mirror_Loop
	;Initialize interrupt counter
	xor a
	ld (people_active),a
	ld (people_killed),a
	ld (people_chopper),a
	ld (people_rescued),a
	ld (blimp_state),a
	dec a
	ld (sortie),a
sortie_start:
	ld hl,bullets
	ld (hl),0
	ld de,bullets+1
	ld bc,(max_bullets*10)-1
	ldir
	bcall(_clrlcdfull)
	ld a,(sortie)
	inc a
	ld (sortie),a
	cp 3
	jp nc,game_over
	ld hl,first_sortie
	or a
	jr z,sortie_start_labelmaker
	ld hl,second_sortie
	dec a
	jr z,sortie_start_labelmaker
	ld hl,third_sortie
sortie_start_labelmaker:
	ld de,2*256+3
	ld (currow),de
	bcall(_puts)
	;Now delay
	ld b,200
sortie_pause:
	ei \ halt
	djnz sortie_pause
	ld hl,objects_initial
	ld de,objects
	ld bc,22
	ldir
	xor a
	ld (counter),a
	ld (lastkey),a
	ld (firetime),a
	ld (base_run),a
	;Helicopter is level
	ld (heliangle),a
	ld (heliblade),a
	ld a,1
	ld (heliface),a
	ld hl,0
	ld (chop_vert),hl
	ld (chop_horz),hl
	;Screen is 1000 pixels wide
	ld hl,(904<<6) | $20
	ld (window_x),hl
Game_Loop:
	;Main game loop
	ld a,(people_rescued)
	ld hl,people_killed
	add a,(hl)
	cp 64
	jp z,game_over
	ei
	halt \ di
	;Check keys
	xor a
	ld (curkey),a
	ld a,$FF
	out (1),a
	nop \ ld a,$FD
	out (1),a
	nop
	in a,(1)
	cp $BF			;CLEAR key
	jp z,Pause
	;Arrow keys
	ld a,$FF
	out (1),a
	nop \ dec a
	out (1),a
	nop
	in a,(1)
	and %00001000
	call z,Chopper_Up
	in a,(1)
	and %00000001
	call z,Chopper_Down
	in a,(1)
	and %00000010
	call z,Chopper_Left
	in a,(1)
	and %00000100
	call z,Chopper_Right
	in a,(1)
	and %00001000
	jr z,KeyS_1
	in a,(1)
	and %00000001
	jr z,KeyS_1
	ld hl,0
	ld (chop_vert),hl
KeyS_1:
	ld a,$FF
	out (1),a
	nop
	ld a,$BF
	out (1),a
	nop	
	in a,(1)
	and %00100000		;2nd key
	call z,Chopper_Fire
	in a,(1)
	cp $BF			;MODE key
	call z,Chopper_Rotate
	ld a,(curkey)
	ld (lastkey),a
	;Cycle through objects and perform any required updates
	ld ix,objects+1
	ld b,(ix-1)
Object_Loop:
	push bc
	ld a,(ix)
	inc (ix+1)
	cp (ix+1)
	jr nz,Object_Skip
	push ix
	ld (ix+1),0
	ld de,Object_Return
	push de
	ld l,(ix+6)
	ld h,(ix+7)
	jp (hl)
Object_Return:
	pop ix
Object_Skip:
	ld de,11
	add ix,de
	pop bc
	djnz Object_Loop
	ld a,(firetime)
	sub 1
	adc a,0
	ld (firetime),a
	ld hl,counter
	inc (hl)
	;Update chopper position left/right
	ld hl,(chop_horz)
	bit 7,h
	jr nz,Sideways_Left
	ld de,(objects+14)
	add hl,de
	ld (objects+14),hl
	or a
	ld de,((1000-23)<<6) | $20
	sbc hl,de
	jr c,Sideways_Right
	ld (objects+14),de
	jr Sideways_Right
Sideways_Left:
	ld a,h \ cpl \ ld d,a
	ld a,l \ cpl \ inc a \ ld e,a
	ld hl,(objects+14)
	or a
	sbc hl,de
	ld de,(16<<6) | $20
	sbc hl,de
	push af
	add hl,de
	pop af
	jr nc,Sideways_Left_1
	ld hl,(16<<6) | $20
Sideways_Left_1:
	ld (objects+14),hl
Sideways_Right:
	ld a,(lastkey)
	cp skLeft
	jr z,Sideways_OK
	cp skRight
	jr z,Sideways_OK
	ld hl,(chop_horz)
	ld d,h
	ld e,l
	add hl,hl
	add hl,hl
	add hl,hl
	or a
	sbc hl,de
	sra h
	rr l
	sra h
	rr l
	sra h
	rr l
	bit 7,h
	jr z,Sideways_Right_2
	inc hl
Sideways_Right_2:
	ld (chop_horz),hl
Sideways_OK:
	ld hl,(objects+16)
	or a
	ld de,((64-ground_height-8)<<6) | $20
	sbc hl,de
	jr nz,Gravity_NotOnGround
	ld hl,0
	ld (chop_horz),hl
Gravity_NotOnGround:
	;Update chopper position for gravity
	ld hl,(objects+16)
	ld de,gravconstant
	add hl,de
	ld de,(chop_vert)
	add hl,de
	bit 7,h
	jr z,Gravity_Top
	ld hl,$0020
Gravity_Top:
	ld (objects+16),hl
	ld de,((64-ground_height-8)<<6) | $20
	sbc hl,de
	jr c,Gravity_Done
	ld (objects+16),de
Gravity_Done:
	;Check the window position...move along with the helicopter
	;Check if the helicopter is too far left
	ld hl,(objects+14)
	ld de,(window_x)
	or a
	sbc hl,de
	add hl,hl
	add hl,hl
	ld l,h
	ld h,0
	ld de,16
	sbc hl,de
	jr c,Window_MoveLeft
	ld de,41
	sbc hl,de
	jr c,Window_Done
	;Move right
	ld hl,(objects+14)
	ld de,(80-23)<<6
	or a
	sbc hl,de
	ld a,l
	and %11000000
	or $20
	ld l,a
	ld (window_x),hl
	jr Window_Done
Window_MoveLeft:
	ld hl,(objects+14)
	ld de,16<<6
	or a
	sbc hl,de
	ld a,l
	and %11000000
	or $20
	ld l,a
	ld (window_x),hl
Window_Done:
	call draw_screen
	jp Game_Loop
draw_screen:
	;First draw the background ground
	ld hl,plotsscreen
	ld (hl),$FF
	ld de,plotsscreen+1
	ld bc,767-(12*ground_height)
	ldir
	inc hl
	inc de
	ld a,(window_x)	;window LSB X position
	add a,a
	add a,a
	ld a,%10101010
	jr nc,ground_norotate
	rlca
ground_norotate:
	ld (hl),a
	ld bc,12*ground_height
	ldir
	;Now objects
	ld ix,objects
	ld b,(ix)
	inc ix
draw_objects:
	push bc
	push ix
	ld l,(ix+9)
	ld h,(ix+10)
	ld de,draw_objects_ret
	push de
	jp (hl)
draw_objects_ret:
	pop ix
	ld bc,11
	add ix,bc
	pop bc
	djnz draw_objects
	;Draw and update people
	ld a,6
	call SetSpriteHeight
	ld hl,people
	ld b,64
draw_people_loop:
	push bc
	push hl
	ld a,(hl)
	add a,a
	jp nc,draw_people_next
	;Check for chopper carry. If so, then we skip the displaying of the hostage image
	;and also check if the chopper set down at base. If it is down, then let the hostages
	;go home! FREE!
	bit 3,a		;shifted by 1, remember?
	jr z,draw_people_miserable
	ld bc,((64-ground_height-8)<<6) | $20
	ld hl,(objects+16)
	or a
	sbc hl,bc
	jp nz,draw_people_next		;Still in the air!
	ld bc,(939<<6) | $20
	ld hl,(objects+14)
	or a
	sbc hl,bc
	jp c,draw_people_next		;too far left...get conservative!
	ld bc,9<<6
	sbc hl,bc
	jp nc,draw_people_next		;too far right...get liberal (ugh)!
	;Goldilocks says: "Just right, Bob!"
	ld a,(base_run)
	dec a
	ld (base_run),a
	inc a
	jp nz,draw_people_next
	ld a,120
	ld (base_run),a
	pop hl
	push hl
	ld a,4
	xor (hl)
	ld (hl),a		;Clear airlifted status
	inc hl
	inc hl
	inc hl
	ex de,hl
	ld hl,(objects+14)
	ld bc,10<<6
	add hl,bc
	ex de,hl
	ld (hl),e
	inc hl
	ld (hl),d
	jp draw_people_next		;We'll draw next time
draw_people_miserable:
	ex af,af'
	ld a,(hl)
	ex af,af'
	;Active person
	inc hl
	dec (hl)
	jr nz,draw_people_loop_LALALA
	;It's zero!
	push af
	ld a,people_update
	ld (hl),a
	dec hl
	ld a,(hl)
	xor %00001000
	ld (hl),a
	inc hl
	pop af
draw_people_loop_LALALA:
	inc hl
	dec (hl)
	jr nz,draw_people_loop_OBAFGKMRNS
	;Zero!! State change time! YAHOO!
	dec hl
	dec hl
	call Person_StateChange
	inc hl
	inc hl
draw_people_loop_OBAFGKMRNS:
	inc hl
	ld e,(hl)
	inc hl
	ld d,(hl)
	;Get state
	and %00000110	;already shifted left by one
	cp 2<<1		;waving?
	jp z,draw_people_draw	;no need to move! Just stay in place, you lazy escapee
	;Check for near-base condition
	push hl
	ld hl,(939<<6) | $20
	or a
	sbc hl,de
	pop hl
	jr nc,draw_people_loop_asdf
	;Close enough!
	ld a,1
	jr draw_people_floatingYESMOO
draw_people_loop_asdf:
	;Are we eligible for parole? I mean, to move...
	push hl
	ld hl,(objects+14)
	or a
	sbc hl,de
	bit 7,h
	jr z,draw_people_liberation
	push af
	ld a,255
	xor h
	ld h,a
	ld a,255
	xor l
	ld l,a
	inc hl
	pop af
	;NEGATED, SIR! For a shiny red mackerel has been plucked from the sea...
draw_people_liberation:
	ld bc,vision_radius
	or a
	sbc hl,bc
	pop hl
	jp nc,draw_people_draw		;we have escaped movement by virtue of our 20/1000 vision
	;Check if the chopper is grounded. If so, run a circuit breaker and wire up 240v.
	;Or actually we then create a mass refugee movement where the little people
	;run for their lives to the chopper.
	push hl
	ld bc,((64-ground_height-8)<<6) | $20
	ld hl,(objects+16)
	or a
	sbc hl,bc
	pop hl
	jr nz,draw_people_floatingYESMOO
	;Tranquility base says the chopper has landed. Run! Run! Run!
	push hl
	ld hl,(objects+14)
	ld bc,10<<6
	add hl,bc
	or a
	sbc hl,de	;compare our position with the chopper's
	;if it matches then rapture
	ld a,h
	or a
	jr nz,draw_people_pablonaruda
	ld a,l
	and %11000000
	jr nz,draw_people_pablonaruda
	;Rapture time
	ld hl,people_chopper
	ld a,(hl)
	cp 16
	jr nc,draw_people_pablonaruda
	inc (hl)
	dec hl
	dec (hl)	;people_active
	;set the appropriate bit
	pop hl
	push hl
	dec hl
	dec hl
	dec hl
	dec hl
	ld a,4
	or (hl)		;set bit 2...currently airlifted
	push af
	ex af,af'
	pop af
	ex af,af'
	ld (hl),a
	pop hl
	jp draw_people_next
draw_people_pablonaruda:
	;if NC, then run right. if C then run left
	ld hl,(objects+14)
	ld bc,10<<6
	add hl,bc
	or a
	sbc hl,de	;compare our position with the chopper's
	pop hl
	push hl
	ccf
	ld a,0
	adc a,a
	ld b,a
	dec hl
	dec hl
	dec hl
	dec hl
	ld a,(hl)
	and %11111100
	or b
	ld (hl),a
	pop hl
	ld a,b
	jr draw_people_randomlabel
draw_people_floatingYESMOO:
	;
	push hl
	ld hl,(970<<6) | $20
	or a
	sbc hl,de
	pop hl
	jr nc,draw_people_randomlabel
	;We're home!
	push hl
	ld hl,people_rescued
	inc (hl)
	ld a,(hl)
	cp 64
	jp z,game_over
	dec hl
	dec (hl)
	pop hl
	dec hl
	dec hl
	dec hl
	dec hl
	ld a,%10010000
	xor (hl)
	ld (hl),a		;deactivate person
	jp draw_people_next
draw_people_randomlabel:
	ld bc,people_velocity
	or a
	jr nz,draw_people_move
	ld bc,65536-people_velocity
draw_people_move:
	ex de,hl
	add hl,bc
	ex de,hl
	;jr c,draw_people_draw
	ld (hl),d
	dec hl
	ld (hl),e
draw_people_draw:
	;DE is the updated position
	ld hl,(invisible_wall<<6) | $20
	or a
	sbc hl,de
	jr nc,draw_people_Mrtiscool
	ld bc,10<<6
	or a
	jr z,draw_people_55
	sbc hl,bc
	jr nc,draw_people_Mrtiscool
draw_people_55:
	;We're bouncing off the walls here!
	;Undo last move!
	ld hl,(invisible_wall-1<<6) | $20
	or a
	jr nz,draw_people_IAMTHEKELPKING
	ld hl,(1<<6) | $20
draw_people_IAMTHEKELPKING:
	ex de,hl
	;or a
	;sbc hl,bc
	;ex de,hl
	pop hl
	push hl
	ld a,1
	xor (hl)
	ld (hl),a
	inc hl
	inc hl
	ld (hl),75
	inc hl
	ld (hl),e
	inc hl
	ld (hl),d
draw_people_mrtiscool:
	ex de,hl
	ld de,(window_x)
	or a
	sbc hl,de
	inc hl
	inc hl
	inc hl
	bit 7,h
	jr nz,draw_people_next
	ld de,(99<<6) | $20
	or a
	sbc hl,de
	jr nc,draw_people_next
	adc hl,de
	inc hl
	add hl,hl
	add hl,hl
	ld b,h
	ld c,64-ground_height-4
	ex af,af'
	push af
	ld hl,People_Sprites
	ld d,0
	and %00000011
	add a,a
	add a,a
	ld e,a
	add hl,de
	pop af
	and %00001000
	rra \ rra
	ld e,a
	add hl,de
	ld a,(hl) \ inc hl \ ld h,(hl) \ ld l,a
	push bc
	call PutSprClp
	pop de
	ld b,e
	ld a,d
	add a,4
	ld e,a
	ld c,64-ground_height+3
	xor a
	call FindCollision
	jr nc,draw_people_next
	pop hl
	ld a,(hl)
	xor %10010000
	ld (hl),a
	push hl
	ld hl,people_active
	dec (hl)
	ld hl,people_killed
	inc (hl)
draw_people_next:
	pop hl
	ld de,5
	add hl,de
	pop bc
	dec b
	jp nz,draw_people_loop
	;Draw and update bullets
	ld ix,bullets
	ld b,max_bullets
draw_bullets_loop:
	push ix
	push bc
	ld a,(ix)
	add a,a
	jp nc,draw_bullets_next
	;Active bullet
	;Update position
	ld l,(ix+1)
	ld h,(ix+2)
	ld e,(ix+5)
	ld d,(ix+6)
	add hl,de
	ld (ix+1),l
	ld (ix+2),h
	push hl
	;Bullet should self-destruct if outside of screen range
	ld de,(window_x)
	or a
	sbc hl,de
	ld de,(96<<6) | $20
	sbc hl,de
	call nc,Bullet_Destruct
	jr c,draw_bullets_tripleblah
	pop hl
	jp draw_bullets_next
draw_bullets_tripleblah:
	ld l,(ix+3)
	ld h,(ix+4)
	ld e,(ix+7)
	ld d,(ix+8)
	ld a,(ix+0)
	and %00001111
	cp 1
	jr z,draw_bullets_blahblah
	cp 4
	jr nz,draw_bullets_halbhalb
	push hl
	;Blimp missile...see if we have movement available?
	ld a,(blimp_byte)
	inc a
	ld (blimp_byte),a
	rra
	jr c,draw_bullets_123zyx
	;Slow X velocity
	ld l,(ix+5)
	ld h,(ix+6)
	ld a,h
	bit 7,a
	ld de,blimp_missilea
	jr z,draw_bullets_halfblah
	ld de,-blimp_missilea
draw_bullets_halfblah:
	or a
	sbc hl,de
	xor h
	add a,a
	jr nc,draw_bullets_blahblah123
	ld (ix),0
	pop hl \ pop hl
	jp draw_bullets_next
draw_bullets_blahblah123:
	ld (ix+5),l
	ld (ix+6),h
draw_bullets_123zyx:
	ld a,(ix+9)
	dec a
	ld (ix+9),a
	bit 7,a
	jr nz,draw_bullets_blahhalb
	ld hl,(objects+16)
	ld de,3<<6
	add hl,de
	ld e,(ix+3)
	ld d,(ix+4)
	or a
	sbc hl,de
	ld de,$20
	jr nc,draw_bullets_halbblah
	ld de,-($20)
	jr draw_bullets_halbblah
draw_bullets_blahhalb:
	ld de,0
draw_bullets_halbblah:
	pop hl
	jr draw_bullets_blahblah
draw_bullets_halbhalb:
	push hl
	ld hl,tank_gravity
	add hl,de
	ld (ix+8),h
	ld (ix+7),l
	ex de,hl
	pop hl
draw_bullets_blahblah:
	add hl,de
	ld (ix+3),l
	ld (ix+4),h
	;Y position is HL
	add hl,hl
	add hl,hl
	ld c,h
	ld a,(ix+8)
	add a,a
	jr c,draw_bullets_bangbang
	ld a,c
	cp 64-ground_height
	call nc,Bullet_Destruct
draw_bullets_bangbang:
	pop hl
	or a
	ld de,(window_x)
	sbc hl,de
	add hl,hl
	add hl,hl
	ld b,h
	;BC is the position
	;Branch based on bullet type
	ld a,(ix+0)
	and %00001111
	cp 4
	jr nz,draw_bullets_2x2
	ld a,3
	call SetSpriteHeight
	ld a,(ix+6)
	add a,a
	ld hl,Missile_Right
	jr nc,draw_bullets_drawyourgunnow
	ld hl,Missile_Left
	jr draw_bullets_drawyourgunnow
draw_bullets_2x2:
	ld a,2
	call SetSpriteHeight
	ld hl,Bullet
draw_bullets_drawyourgunnow:
	call PutSprClp
draw_bullets_next:
	ld de,10
	pop bc
	pop ix
	add ix,de
	dec b
	jp nz,draw_bullets_loop
	jp ionfastcopy
objects_initial:
	.db 9	;object count
	;Format:
	;Update interval (DB), Update counter (DB) X position (DW), Y position (DW), Update pointer (DW), State flags (DB), Display pointer (DW)
	;The helicopter and base *must* be left in this order
	;at the beginning of the objects list, or else the code will break.
	;Base
	.db 9,0
	.dw (939<<6) | $20
	.dw ((64-ground_height-19)<<6) | $20
	.dw U_Base
	.db 0
	.dw D_Base
	;User's helicopter
	.db 7,0
	.dw (955<<6) | $20
	.dw (5<<6) | $20
	.dw U_Helicopter
	.db 0
	.dw D_Helicopter
	;House 0
	.db 7,0
	.dw (house0<<6) | $20
	.db 0,0		;first byte here is a counter until next hostage release
	.dw U_House
	.db %00000000	;bit 7 - if set, then burned (open), bit 4 is the flame state (0-1)
	;bit 5-6 is the house number
	.dw D_House
	;House 1
	.db 7,0
	.dw (house1<<6) | $20
	.db 0,0
	.dw U_House
	.db %00100000
	.dw D_House
	;House 2
	.db 7,0
	.dw (house2<<6) | $20
	.db 0,0
	.dw U_House
	.db %01000000
	.dw D_House
	;House 3
	.db 7,0
	.dw (house3<<6) | $20
	.db 0,0
	.dw U_House
	.db %01100000
	.dw D_House
	;Tank
	.db 6,0
	.dw (500<<6) | $20
	.db 1,0		;first byte on this line is the countdown until we can fire again
	;Second byte on that line is the current attack state. 0 through 2 are various distances where we want to be from the helicopter so that the lobbed bullet hits it.
	.dw U_Tank
	.db 0
	.dw D_Tank
	;EVIL BLIMP!
	.db 6,0
	.dw (0<<6) | $20
	.dw (30<<6) | $20
	.dw U_Blimp
	.db 15		;countdown until blimp appears again (or can fire again)
	.dw D_Blimp
	;Border
	.db 0,0
	.dw (invisible_wall+10)<<6 | $20
	.dw 0
	.dw U_Border
	.db 0
	.dw D_Border
objects_end:
D_Border:
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(window_x)
	ld bc,(24<<6) | $20
	add hl,bc
	or a
	sbc hl,de
	ret c
	ld l,(ix+2)
	ld h,(ix+3)
	or a
	sbc hl,de
	jr c,D_Border_2
	ld de,(96<<6) | $20
	or a
	sbc hl,de
	ret nc
	add hl,de
D_Border_2:
	ld a,19
	call SetSpriteHeight
	add hl,hl
	add hl,hl
	ld b,h
	ld c,64-ground_height-19+2
	ld hl,Border
	ld e,3
D_Border_1:
	push de
	push bc
	push hl
	push ix
	call PutSprClp
	pop ix
	pop hl
	pop bc
	ld a,8
	add a,b
	ld b,a
	ld de,19*2
	add hl,de
	pop de
	dec e
	jr nz,D_Border_1
	ret
U_Blimp:
	ld a,(people_rescued)
	or a
	ret z		;no evil drigibles on the first sortie!
	ld a,(blimp_state)
	or a
	jp z,U_Blimp_Inactive
	;Check if crashing. If so no x adjustment but plummet down to a fiery death instead.
	cp 1
	jr nz,U_Blimp_Alive
	dec (ix+8)
	ld a,(ix+8)
	dec a
	and %00001111
	ret nz
	ld a,(ix+8)
	sub 16-blimp_dead
	ld (ix+8),a
	and %11110000
	ret nz
	ld (blimp_state),a
U_Border:
	ret
U_Blimp_Alive:
	;Blimp is active...Make adjustment for x position
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,blimp_speed
	rra
	jr c,U_Blimp_Right
	ld de,-blimp_speed
U_Blimp_Right:
	add hl,de
	ld bc,invisible_wall<<6
	or a
	sbc hl,bc
	jr c,U_Blimp_FreeRangeChicken
	or a
	sbc hl,de
	ld a,(blimp_state)
	xor 1
	ld (blimp_state),a
U_Blimp_FreeRangeChicken:
	add hl,bc
	ld (ix+2),l
	ld (ix+3),h
	;Wind drift?
	ld b,70
	call ionRandom
	sub 35
	ld e,a
	add a,a
	sbc a,a
	ld d,a
	ld l,(ix+4)
	ld h,(ix+5)
	add hl,de
	bit 7,h
	ret nz
	ld de,(64-ground_height-8-20)<<6 | $20
	or a
	sbc hl,de
	ret nc
	add hl,de
	ld (ix+4),l
	ld (ix+5),h
	;Fire missile?
	ld a,(ix+8)
	dec a
	ld (ix+8),a
	ret nz
	ld (ix+8),blimp_reloadtime
	;Fire!
	call Find_Free_Bullet
	ret c
	ld (hl),$84		;active missile!
	inc hl
	ex de,hl
	ld l,(ix+2)
	ld h,(ix+3)
	ld bc,8<<6
	add hl,bc
	ex de,hl
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	ex de,hl
	ld l,(ix+4)
	ld h,(ix+5)
	ld bc,7<<6
	add hl,bc
	ex de,hl
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	;velocity
	push hl
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(objects+14)
	or a
	sbc hl,de
	ld de,blimp_missilev
	jr c,U_Blimp_FIREINTHEHOLE
	ld de,-blimp_missilev
U_Blimp_FIREINTHEHOLE:
	pop hl
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	ld (hl),0
	inc hl
	ld (hl),0
	inc hl
	ld (hl),blimp_missilem
	ret
U_Blimp_Inactive:
	ld a,(ix+8)
	dec a
	ld (ix+8),a
	ret nz
	;Time to activate!
	ld b,50
	call ionRandom
	add a,25
	ld (ix+8),a
	ld a,3
	ld (blimp_state),a
U_Blimp_JumpIn:
	ld b,(invisible_wall-25)/4
	call ionRandom
	;Shift 6 times, then multiply by 4.
	ld e,0
	ld d,a
	ld hl,(window_x)
	ld bc,65536-(23<<6)
	add hl,bc
	or a
	sbc hl,de
	jr nc,U_Blimp_BLIMPIESUBS
	ld hl,(window_x)
	ld bc,96<<6
	add hl,bc
	or a
	sbc hl,de
	jr nc,U_Blimp_JumpIn
U_Blimp_BLIMPIESUBS:
	ld (ix+2),e
	ld (ix+3),d
	ld hl,(20<<6) | $20
	ld (ix+4),l
	ld (ix+5),h
	ret
D_Blimp:
	ld a,(blimp_state)
	or a
	ret z
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(window_x)
	ld bc,(21<<6) | $20
	add hl,bc
	or a
	sbc hl,de
	jp c,D_Blimp_OffScreen
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(window_x)
	or a
	sbc hl,de
	jr c,D_Blimp_2
	ld de,(96<<6) | $20
	or a
	sbc hl,de
	jr nc,D_Blimp_OffScreen
	add hl,de
D_Blimp_2:
	add hl,hl
	add hl,hl
	ld b,h
	ld l,(ix+4)
	ld h,(ix+5)
	add hl,hl
	add hl,hl
	ld c,h
	ld a,(blimp_state)
	cp 1
	jr nz,D_Blimp_pmilb
	ld a,(ix+8)
	and %11110000
	rrca \ rrca \ rrca
	ld e,a
	ld d,0
	ld hl,Blimp_Explosion_Sprites
	add hl,de
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	jr D_Blimp_BLIMPIE
D_Blimp_pmilb:
	rra
	ld hl,Blimp_Left
	jr nc,D_Blimp_BLIMPIE
	ld hl,Blimp_Right
D_Blimp_BLIMPIE:
	ld e,3
	ld a,12
	call SetSpriteHeight
	push bc
D_Blimp_1:
	push de
	push bc
	push hl
	push ix
	call PutSprClp
	pop ix
	pop hl
	pop bc
	ld a,8
	add a,b
	ld b,a
	ld de,24
	add hl,de
	pop de
	dec e
	jr nz,D_Blimp_1
	pop de
	ld b,e
	ld a,21
	add a,d
	ld e,a
	ld a,13
	add a,b
	ld c,a
	ld a,%10000011
	call FindCollision
	ret nc
	;Collision!
	ld a,1
	ld (blimp_state),a
	ld (ix+8),$20 | blimp_dead
	ret
D_Blimp_OffScreen:
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(objects+14)
	or a
	sbc hl,de
	ld a,1
	adc a,a
	ld hl,invisible_wall<<6
	or a
	sbc hl,de
	ret c
	ld (blimp_state),a
	ret
U_Tank:
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,65536-((10<<6) | $20)
	add hl,de
	ld de,(objects+14)
	or a
	sbc hl,de
	ld c,0
	;C if we are to the left of the helicopter
	push af
	push af
	ex af,af'
	pop af
	ex af,af'
	rl c		;C is 1 if we want to move right. 0 if we want to move left.
	pop af
	jr nc,U_Tank_GO
	bit 7,h
	jr z,U_Tank_GO
	;Negate!
	ld a,h
	cpl
	ld h,a
	ld a,l
	cpl
	ld l,a
	inc hl
U_Tank_GO:
	push hl
	ld a,(ix+5)
	add a,a
	ld d,0
	ld e,a
	ld hl,Tank_Bullet_Distances
	add hl,de
	ld e,(hl)
	inc hl
	ld d,(hl)
	pop hl
	;DE is the distance away from the helicopter we want to be
	or a
	sbc hl,de
	push af
	add hl,hl
	add hl,hl
	ld a,h
	or a
	jr nz,U_Tank_MOOOO
	pop af
	jr U_Tank_Done
U_Tank_MOOOO:
	pop af
	jr nc,U_Tank_ME_Tank
	ld a,1
	xor c
	ld c,a
U_Tank_ME_Tank:
	ld a,c
	or a
	ld de,tank_velocity
	jr nz,U_Tank_ME_Tank_WEALLTANK
	ld de,65536-tank_velocity
U_Tank_ME_Tank_WEALLTANK:
	ld l,(ix+2)
	ld h,(ix+3)
	ld bc,(invisible_wall<<6) | $20
	or a
	sbc hl,bc
	jr nc,U_Tank_MOOO
	add hl,bc
	add hl,de
	ld (ix+2),l
	ld (ix+3),h
U_Tank_Done:
	;Now check if we can fire a bullet!
	ld a,(ix+4)
	dec a
	ld (ix+4),a
	ret nz
	;We can fire!
	ld a,tank_reload
	ld (ix+4),a
	call Find_Free_Bullet
	jr c,U_Tank_ShotThatChangedTheWorld
	ld (hl),%10000010
	inc hl
	;X position?
	push hl
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(4<<6) | $20
	add hl,de
	ex de,hl
	pop hl
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	;Y position?
	ld de,((64-6)<<6) | $20
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	push hl
	ld hl,Tank_Bullet_XVelocity
	ld a,(ix+5)
	add a,a
	ld d,0
	ld e,a
	add hl,de
	ld e,(hl)
	inc hl
	ld d,(hl)
	pop hl
	ex af,af'
	jr c,U_Tank_MOOOOOO
	ld a,d
	cpl
	ld d,a
	ld a,e
	cpl
	ld e,a
	inc de
U_Tank_MOOOOOO:
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	push hl
	ld hl,Tank_Bullet_YVelocity
	ld a,(ix+5)
	add a,a
	ld d,0
	ld e,a
	add hl,de
	ld e,(hl)
	inc hl
	ld d,(hl)
	pop hl
	ld (hl),e
	inc hl
	ld (hl),d
U_Tank_ShotThatChangedTheWorld:
	;Now we're done so randomly select a new fire distance and angle
	ld b,3
	call ionRandom
	ld (ix+5),a
	ret
U_Tank_MOOO:
	ld hl,invisible_wall<<6
	ld (ix+2),l
	ld (ix+3),h
	jr U_Tank_Done
Tank_Bullet_XVelocity:
	;0,1,2
	;cos(pi/4)
	.dw (1<<6) | $00
	.dw (0<<6) | $20
	.dw (1<<6) | $30
Tank_Bullet_YVelocity:
	.dw 65536-((1<<6) | $00)
	.dw 65536-((1<<6) | $10)
	.dw 65536-((1<<6) | $00)
Tank_Bullet_Distances:
	;0,1,2
	.dw (30<<6) | $20
	.dw (16<<6) | $20
	.dw (60<<6) | $20
D_Tank:
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(window_x)
	ld bc,(8<<6) | $20
	add hl,bc
	or a
	sbc hl,de
	ret c
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(window_x)
	or a
	sbc hl,de
	jr c,D_Tank_1
	ld de,(96<<6) | $20
	or a
	sbc hl,de
	ret nc
	add hl,de
D_Tank_1:
	ld a,5
	call SetSpriteHeight
	add hl,hl
	add hl,hl
	ld b,h
	ld c,64-6
	push bc
	push ix
	ld hl,Tank
	call PutSprClp
	pop ix
	pop de
	ld a,9
	add a,d
	ld e,a
	ld b,64-7
	ld c,65
	ld a,%10000001
	call FindCollision
	ret nc
	;Collision...respawn off screen
D_Tank_RandomNUMBARBABY:
	ld b,(invisible_wall-10)/3
	call ionRandom
	;Shift 3 times to get back to pixels. Then 6 more for a total of 9.
	ld l,0
	ld h,a
	add hl,hl
	ex de,hl
	ld hl,(window_x)
	or a
	sbc hl,de
	push af
	add hl,de
	pop af
	jr nc,D_Tank_IsInDWorld
	ld bc,(96<<6) | $20
	add hl,bc
	or a
	sbc hl,de
	jr c,D_Tank_IsInDWorld
	jr D_Tank_RandomNUMBARBABY
D_Tank_IsInDWorld:
	ld (ix+2),e
	ld (ix+3),d
	ret
U_House:
	ld a,(ix+8)
	add a,a
	ret nc
	ld a,%00010000
	xor (ix+8)
	ld (ix+8),a
	;Release a hostage?
	ld a,(ix+4)
	or a
	ret z
	dec a
	ld (ix+4),a
	ret nz
	ld a,hostage_time
	ld (ix+4),a
	;Release a hostage!
	ld hl,people
	ld a,(ix+8)
	and %01100000
	ld c,a
	ld b,64
	ld de,5
U_House_HostageScan:
	ld a,(hl)
	and %01100000
	cp c
	jr nz,U_House_HostageScan_Next
	;Match
	ld a,(hl)
	add a,a
	jr c,U_House_HostageScan_Next	;hostage already active?
	bit 5,a
	jr nz,U_House_HostageScan_Next	;RIP
	;How many are out?
	ld a,(people_active)
	cp 16
	jr nc,U_House_HostageScan_Next
	;Let's cast out the hostage!
	ld a,(hl)
	or %10000001		;state 1
	ld (hl),a
	call Person_StateChange
	push hl
	ld hl,people_active
	inc (hl)
	pop hl
	ret
U_House_HostageScan_Next:
	add hl,de
	djnz U_House_HostageScan
	ret
D_House:
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(window_x)
	ld bc,(23<<6) | $20
	add hl,bc
	or a
	sbc hl,de
	ret c	;House is off the left edge of the screen
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(window_x)
	or a
	sbc hl,de
	jr c,D_House_2
	ld de,(96<<6) | $20
	or a
	sbc hl,de
	ret nc	;House is off the right edge of the screen
	add hl,de
D_House_2:
	ld a,12
	call SetSpriteHeight
	add hl,hl
	add hl,hl
	ld b,h
	ld c,64-ground_height-12+2
	ld a,(ix+8)
	add a,a
	ld hl,House_Closed
	jr nc,D_House_OK
	ld hl,House_Open1
	and %00100000
	jr nz,D_House_OK
	ld hl,House_Open2
D_House_OK:
	ld e,3
D_House_1:
	push de
	push bc
	push hl
	push ix
	call PutSprClp
	pop ix
	pop hl
	pop bc
	ld a,8
	add a,b
	ld b,a
	ld de,24
	add hl,de
	pop de
	dec e
	jr nz,D_House_1
	;Check for bullet collisions
	ld a,(ix+8)
	add a,a
	ret c	;already open
	exx
	ld l,(ix+2)
	ld h,(ix+3)
	ld de,(window_x)
	or a
	sbc hl,de
	add hl,hl
	add hl,hl
	ld d,h		;D stores the left edge
	ld a,23
	add a,h
	ld e,a		;E stores the right edge + 1
	ld b,64-ground_height-12+2	;B stores the top edge
	ld a,12
	add a,b
	ld c,a		;C stores the bottom edge + 1
	xor a
	call FindCollision
	ret nc
	;Collision!
	ld a,(ix+8)
	add a,a
	ret c	;already open
	scf
	rra
	ld (ix+8),a
	ld a,hostage_time
	ld (ix+4),a
	ret
U_Base:
	;Toggle flag
	ld a,80h
	xor (ix+8)
	ld (ix+8),a
	ret
U_Helicopter:
	;First update rotor animation
	ld a,(heliblade)
	inc a
	cp 3
	jr nz,U_Helicopter_1
	xor a
U_Helicopter_1:
	ld (heliblade),a
	ret
D_Helicopter:
	;Draw the helicopter
	;We take into consideration the angle of the helicopter,
	;the direction it faces, and the status of the rotors
	ld a,(heliface)
	ld hl,Sprite_Helicopter_Center
	or a
	jr z,D_Helicopter_1
	ld hl,Sprite_Helicopter_Right
	dec a
	jr z,D_Helicopter_1
	ld hl,Sprite_Helicopter_Left
D_Helicopter_1:
	ld a,(heliangle)
	ld b,a
	add a,a
	add a,b
	;x3
	ld b,a
	ld a,(heliblade)
	add a,b
	add a,a
	ld e,a
	ld d,0
	add hl,de
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	ld a,(heliface)
	cp 2
	jr nz,D_Helicopter_4
	ld de,32
	add hl,de
D_Helicopter_4:
	;pointer to sprite
	ld a,8
	call SetSpriteHeight
	push hl
	ld hl,(objects+14)
	ld de,(window_x)
	or a
	sbc hl,de
	;Convert to integer
	add hl,hl
	add hl,hl
	ld b,h
	ld hl,(objects+16)
	add hl,hl
	add hl,hl
	ld c,h
	pop hl
	ld e,3
D_Helicopter_2:
	push de
	push bc
	push hl
	call PutSprClp
	pop hl
	pop bc
	ld a,8
	add a,b
	ld b,a
	ld a,(heliface)
	ld de,16
	cp 2
	jr nz,D_Helicopter_3
	ld de,-16
D_Helicopter_3:
	add hl,de
	pop de
	dec e
	jr nz,D_Helicopter_2
	;Check for collision
	ld hl,(objects+14)
	ld de,(window_x)
	or a
	sbc hl,de
	add hl,hl
	add hl,hl
	ld a,h
	ld d,a
	add a,23
	ld e,a
	ld a,(heliface)
	or a
	jr nz,D_Helicopter_FLYAWAY
	ld a,e
	sub 7
	ld e,a
	ld a,d
	add a,7
	ld d,a
D_Helicopter_FLYAWAY:
	ld hl,(objects+16)
	add hl,hl
	add hl,hl
	ld a,h
	ld b,a
	add a,7
	ld c,a
	ld a,%10001110
	call FindCollision
	ret nc
	;Collision!
	pop hl
	pop hl
	pop hl
	pop hl
	jp chopper_crash
D_Base:
	ld hl,(window_x)
	ld de,(95<<6) | $20
	add hl,de
	ld c,(ix+2)
	ld b,(ix+3)
	or a
	sbc hl,bc
	ret c		;Base is not visible
	ld h,b
	ld l,c
	ld bc,(window_x)
	sbc hl,bc
	add hl,hl
	add hl,hl
	ld b,h
	ld l,(ix+4)
	ld h,(ix+5)
	add hl,hl
	add hl,hl
	ld c,h
	ld a,24
	call SetSpriteHeight
	ld e,8
	ld a,(ix+8)
	rla
	jr c,D_Base_Flag
	ld hl,Sprite_Base_0
	jr D_Base_Loop
D_Base_Flag:
	ld hl,Sprite_Base_1
D_Base_Loop:
	push de
	push hl
	push bc
	call PutSprClp
	pop bc
	ld a,8
	add a,b
	ld b,a
	pop hl
	ld de,48
	add hl,de
	pop de
	dec e
	jr nz,D_Base_Loop
	ret
Chopper_Down:
	ld de,(chop_vert)
	ld hl,y_acceleration
	add hl,de
	ld (chop_vert),hl
	ld a,skDown
	ld (curkey),a
	ld de,gravconstant*2
	scf
	sbc hl,de
	ret c
	ld (chop_vert),de
	ret
Chopper_Up:
	ld de,(chop_vert)
	ld hl,65536-y_acceleration
	add hl,de
	ld (chop_vert),hl
	ld a,skUp
	ld (curkey),a
	ld de,65536-(gravconstant*5/2)
	or a
	sbc hl,de
	ret nc
	ld (chop_vert),de
	ret
Chopper_Left:
	ld hl,(objects+16)
	or a
	ld de,((64-ground_height-8)<<6) | $20
	sbc hl,de
	ret z
	ld de,(chop_horz)
	ld hl,65536-x_acceleration
	add hl,de
	ld (chop_horz),hl
	ld a,skLeft
	ld (curkey),a
	;Check if we are moving too fast leftwards
	ld de,65536-(sidewaysv)
	or a
	sbc hl,de
	ret nc
	ld (chop_horz),de
	ret
Chopper_Right:
	ld hl,(objects+16)
	or a
	ld de,((64-ground_height-8)<<6) | $20
	sbc hl,de
	ret z
	ld de,(chop_horz)
	ld hl,x_acceleration
	add hl,de
	ld (chop_horz),hl
	ld a,skRight
	ld (curkey),a
	;Too fast rightwards?
	ld de,sidewaysv
	scf
	sbc hl,de
	ret c
	ld (chop_horz),de
	ret
Chopper_Fire:
	;Helicopter to base, requesting clearance to engage
	;the enemy, over.
	ld a,(firetime)
	or a
	ret nz		;Negative, chopper. Magic 8-ball says "ask again later." Out.
	;Roger, cleared to fire primary weapon system.
	;Do we have a free bullet (world maximum of 16)?
	call Find_Free_Bullet
	ret c
	ld a,reload_time
	ld (firetime),a
	ld (hl),$81
	;.dw x position, y position, x velocity, y velocity
	inc hl
	push hl
	ld a,(heliface)
	add a,a
	add a,a
	add a,a
	ld c,a
	ld b,0
	ld ix,Chopper_Fire_Table
	add ix,bc
	ld hl,(objects+14)
	ld e,(ix)
	ld d,(ix+1)
	add hl,de
	ex de,hl
	pop hl
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	push hl
	ld hl,(objects+16)
	ld e,(ix+2)
	ld d,(ix+3)
	add hl,de
	ex de,hl
	pop hl
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	ld e,(ix+4)
	ld d,(ix+5)
	ld (hl),e
	inc hl
	ld (hl),d
	inc hl
	ld e,(ix+6)
	ld d,(ix+7)
	ld (hl),e
	inc hl
	ld (hl),d
	ret
#define bullet_speed 2<<6
Chopper_Fire_Table:
	;x offset, y offset, x velocity, y velocity
	;Center
	.dw 11<<6, 5<<6, 0, bullet_speed
	;Right
	.dw 21<<6, 3<<6, bullet_speed, 0
	;Left
	.dw 0, 3<<6, 65536-bullet_speed, 0
Find_Free_Bullet:
	;Output - HL points to a free bullet slot (10 bytes)
	;Returns C if could not find a bullet
	ld hl,bullets
	ld de,10
	ld b,max_bullets
Find_Free_Bullet_Loop:
	ld a,(hl)
	add a,a
	ret nc
	add hl,de
	djnz Find_Free_Bullet_Loop
	scf
	ret
Chopper_Rotate:
	;0 - center
	;1 - right
	;2 - left
	ld a,skMode
	ld (curkey),a
	ld a,(lastkey)
	cp skMode
	ret z
	ld b,0
	ld a,(heliface)
	or a
	jr nz,Chopper_Rotate_Done
	inc b
	ld a,(chop_horz+1)
	add a,a
Chopper_Rotate_Done:
	ld a,b
	adc a,0
	ld (heliface),a
	ret
Pause:
	ld a,skClear
	ld (curkey),a
	ld hl,4*256+3
	ld (currow),hl
	ld hl,Pause_Message
	bcall(_puts)
	res indiconly,(iy+indicflags)
	ei
Pause_Loop:
	bcall(_getcsc)
	cp skClear
	jr z,Pause_Loop
	ei \ halt
	bcall(_getkey)
	set indiconly,(iy+indicflags)
	cp kMode
	jp nz,Game_Loop
	jp game_over
Pause_Message:
	.db " PAUSED ",0
FindCollision:
	;Input - A contains a mask. If bit 7, then match only certain bullets.
	;Lower nibble contains mask with 1 for bits to match
	;Input - shadow registers swapped in
	;
	push ix
	exx
	ex af,af'
	ld ix,bullets
	ld de,8
	ld c,0
	ld b,max_bullets
Collision_Loop:
	ld a,(ix)
	add a,a
	jr nc,Collision_Hollow
	ex af,af'
	ld h,a
	ex af,af'
	bit 7,h
	jr z,Collision_stillgood
	;Mask!
	sla h
	and %00011111
	and h
	jr z,Collision_Hollow
Collision_stillgood:
	;Active bullet
	and %00011110
	ld hl,3*256+3		;H width; L height
	cp 4<<1
	jr nz,Collision_Bullet0	;chopper & tank bullets are 2x2
	ld hl,4*256+3		;Blimp missiles are 4x3
Collision_Bullet0:
	ld (collision_height),hl
	;The normal registers are active
	exx
	ld hl,collision_width
	ld a,d
	sub (hl)
	ld l,(ix+1)
	ld h,(ix+2)
	push de
	ld de,(window_x)
	or a \ sbc hl,de
	ld de,127<<6
	push hl
	add hl,de
	ld de,255<<6
	or a
	sbc hl,de
	pop hl
	pop de
	jr nc,Collision_No
	add hl,hl \ add hl,hl
	bit 7,a
	jr nz,Collision_Crazy
	cp h
	jr nc,Collision_No
	;left boundary checked
Collision_Crazy:
	;now check right
	ld a,e
	cp h
	jr c,Collision_No
	;now top boundary
	ld hl,collision_height
	ld a,b
	sub (hl)
	ld l,(ix+3)
	ld h,(ix+4)
	add hl,hl \ add hl,hl
	cp h
	jr nc,Collision_No
	;now bottom boundary
	ld a,c
	cp h
	jr c,Collision_No
	exx
	ld c,$FF
	call Bullet_Destruct
	exx
Collision_No:
	exx 
Collision_Hollow:
	add ix,de
	djnz Collision_Loop
	sla c	;C if there was a collision
	pop ix
	ret
Sprite_Helicopter_Left:
	.dw Sprite_Helicopter_Left_s0
	.dw Sprite_Helicopter_Left_s1
	.dw Sprite_Helicopter_Left_s2
Sprite_Helicopter_Center:
	.dw Sprite_Helicopter_Center_s0
	.dw Sprite_Helicopter_Center_s1
	.dw Sprite_Helicopter_Center_s2
Sprite_Helicopter_Right:
	.dw Sprite_Helicopter_Right_a0s0
	.dw Sprite_Helicopter_Right_a0s1
	.dw Sprite_Helicopter_Right_a0s2
Sprite_Helicopter_Right_a0s0:
	;Three sections for a0s0
	;Width: 23 pixels, 8 pixel height
	.db %00000000
	.db %11111111
	.db %11111111
	.db %10001111
	.db %11000000
	.db %11100000
	.db %11111111
	.db %11111100
	;sprite
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	;Section 2
	.db %00000000
	.db %11000011
	.db %00000000
	.db %10000000
	.db %00000000
	.db %00000000
	.db %10011100
	.db %00000000
	;sprite
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	;Section 3
	.db %00000001
	.db %11111111
	.db %00000111
	.db %00000011
	.db %00000011
	.db %00000111
	.db %11111111
	.db %00000111
	;sprite
	.db %00000000
	.db %00000000
	.db %00000000
	.db %11110000
	.db %01111000
	.db %00000000
	.db %00000000
	.db %00000000
Sprite_Helicopter_Right_a0s1:
	;Width: 23 pixels, 8 pixel height
	.db %00000000
	.db %11111111
	.db %10111111
	.db %11011111
	.db %11000000
	.db %11100000
	.db %11111111
	.db %11111100
	;sprite
	.db %11111111
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	;Section 2
	.db %10000000
	.db %11000011
	.db %00000000
	.db %10000000
	.db %00000000
	.db %00000000
	.db %10011100
	.db %00000000
	;sprite
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	;Section 3
	.db %00000001
	.db %11111111
	.db %00000111
	.db %00000011
	.db %00000011
	.db %00000111
	.db %11111111
	.db %00000111
	;sprite
	.db %00000000
	.db %00000000
	.db %00000000
	.db %11110000
	.db %01111000
	.db %00000000
	.db %00000000
	.db %00000000
Sprite_Helicopter_Right_a0s2:
;Width: 23 pixels, 8 pixel height
	.db %00000000
	.db %11111111
	.db %11011111
	.db %11011111
	.db %11000000
	.db %11100000
	.db %11111111
	.db %11111100
	;sprite
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	;Section 2
	.db %00000011
	.db %11000011
	.db %00000000
	.db %10000000
	.db %00000000
	.db %00000000
	.db %10011100
	.db %00000000
	;sprite
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	;Section 3
	.db %11111111
	.db %11111111
	.db %00000111
	.db %00000011
	.db %00000011
	.db %00000111
	.db %11111111
	.db %00000111
	;sprite
	.db %00000000
	.db %00000000
	.db %00000000
	.db %11110000
	.db %01111000
	.db %00000000
	.db %00000000
	.db %00000000
Sprite_Helicopter_Center_s0:
	;23x8
	;mask
	.db %00000000
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111110
	;sprite
	.db 0,0,0,0,0,0,0,0
	;mask
	.db %00000000
	.db %11000111
	.db %10000011
	.db %00000001
	.db %00000001
	.db %10000011
	.db %00111001
	.db %01111100
	;sprite
	.db 0,0,0
	.db %01111100
	.db 0,0,0,0
	;mask
	.db %00000001
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0
Sprite_Helicopter_Center_s1:
	;23x8
	;mask
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111110
	;sprite
	.db 0,0,0,0,0,0,0,0
	;mask
	.db %10000000
	.db %11000111
	.db %10000011
	.db %00000001
	.db %00000001
	.db %10000011
	.db %00111001
	.db %01111100
	;sprite
	.db 0,0,0
	.db %01111100
	.db 0,0,0,0
	;mask
	.db %00000001
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0
Sprite_Helicopter_Center_s2:
	;23x8
	;mask
	.db %00000000
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111110
	;sprite
	.db 0,0,0,0,0,0,0,0
	;mask
	.db %00000011
	.db %11000111
	.db %10000011
	.db %00000001
	.db %00000001
	.db %10000011
	.db %00111001
	.db %01111100
	;sprite
	.db 0,0,0
	.db %01111100
	.db 0,0,0,0
	;mask
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0
Sprite_Base_0:
	;Flag up
	;58x24
	;Section 1
	;Mask first
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %01010010
	.db %01001010
	.db %01001010
	.db %00101010
	.db %00000000
	;Section 2
	;mask
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %10101010
	.db %10101010
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 3
	;mask
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;	
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %10101010
	.db %10101010
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 4
	;mask
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0	
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11110000
	.db %11100000
	.db %11101010
	.db %11101000
	.db %11101011
	.db %11101000
	.db %11101010
	.db %11101010
	.db %10000000
	.db %00000000
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 5
	;mask
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %00000000
	.db %00000000
	.db %10111110
	.db %00111110
	.db %10101110
	.db %00111110
	.db %10111110
	.db %10111110
	.db %00000000
	.db %00000000
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 6
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db %00101111
	.db %00101010
	.db %00101010
	.db %00001000
	.db %00010001
	.db %00000100
	.db %00010001
	.db %00010101
	.db %00111111
	.db %00111111
	.db %00111111
	.db %00000111
	.db %00000011
	.db %10101011
	.db %00001011
	.db %11101011
	.db %00001011
	.db %10101011
	.db %10101011
	.db %00000000
	.db %00000000
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 7
	;mask
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db %01110111
	.db %01000111
	.db %00010111
	.db %01000111
	.db %00010111
	.db %01000111
	.db %00010111
	.db %11011111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %00101001
	.db %00001001
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 8
	;mask
	.db 63,63,63,63,63,63,63,63,63,63,63,63
	.db 63,63,63,63,63,63,63,63,63,63,63,63
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %01000000
	.db %01000000
	.db %01000000
	.db %01000000
	.db %00000000
Sprite_Base_1:
	;Flag down
	;58x24
	;Section 1
	;Mask first
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %01010010
	.db %01001010
	.db %01001010
	.db %00101010
	.db %00000000
	;Section 2
	;mask
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %10101010
	.db %10101010
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 3
	;mask
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;	
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %10101010
	.db %10101010
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 4
	;mask
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0	
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11110000
	.db %11100000
	.db %11101010
	.db %11101000
	.db %11101011
	.db %11101000
	.db %11101010
	.db %11101010
	.db %10000000
	.db %00000000
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 5
	;mask
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %00000000
	.db %00000000
	.db %10111110
	.db %00111110
	.db %10101110
	.db %00111110
	.db %10111110
	.db %10111110
	.db %00000000
	.db %00000000
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 6
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db %00101011
	.db %00101010
	.db %00101010
	.db %00100010
	.db %00000100
	.db %00010001
	.db %00000100
	.db %00010111
	.db %00111111
	.db %00111111
	.db %00111111
	.db %00000111
	.db %00000011
	.db %10101011
	.db %00001011
	.db %11101011
	.db %00001011
	.db %10101011
	.db %10101011
	.db %00000000
	.db %00000000
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 7
	;mask
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	.db %11011111
	.db %00010111
	.db %01000111
	.db %00010111
	.db %01000111
	.db %00010111
	.db %01000111
	.db %01110111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %00101001
	.db %00001001
	.db %10101010
	.db %10101010
	.db %00000000
	;Section 8
	;mask
	.db 63,63,63,63,63,63,63,63,63,63,63,63
	.db 63,63,63,63,63,63,63,63,63,63,63,63
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %11000000
	.db %01000000
	.db %01000000
	.db %01000000
	.db %01000000
	.db %00000000
House_Closed:
	;23x12
	;section 1
	;mask
	.db %10101111
	.db %10101100
	.db %11100001
	.db %11000101
	.db %00010101
	.db %01010101
	.db %01010101
	.db %01010100
	.db %01010001
	.db %01000101
	.db %00000000
	.db %00000000
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 2
	;mask
	.db %00000001
	.db %01111100
	.db %01111101
	.db %01011101
	.db %01111101
	.db %01111101
	.db %00000001
	.db %00010001
	.db %00010001
	.db %00010001
	.db %00000000
	.db %00000000
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 3
	;mask
	.db %11111111
	.db %01111111
	.db %00011111
	.db %01000111
	.db %01010001
	.db %01010101
	.db %01010101
	.db %01010101
	.db %01010101
	.db %01010101
	.db %00000001
	.db %00000001
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
House_Open1:
	;23x12
	;section 1
	;mask
	.db %10101111
	.db %10101100
	.db %11100001
	.db %11000101
	.db %00010101
	.db %01010100
	.db %01010111
	.db %01010111
	.db %01010010
	.db %01000111
	.db %00000000
	.db %00000000
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0
	.db %00000001
	.db 0
	;section 2
	;mask
	.db %00000001
	.db %01111100
	.db %01111101
	.db %11111101
	.db %11111101
	.db %11111100
	.db %01111110
	.db %11001111
	.db %01010001
	.db %01011101
	.db %00000000
	.db %00000000
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0
	.db %01010110
	.db 0
	;section 3
	;mask
	.db %11111111
	.db %01111111
	.db %00011111
	.db %01000111
	.db %01010001
	.db %11010101
	.db %11010101
	.db %10010101
	.db %11010101
	.db %10010101
	.db %00000001
	.db %00000001
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
House_Open2:
;23x12
	;section 1
	;mask
	.db %10101111
	.db %10101100
	.db %11100001
	.db %11000101
	.db %00010101
	.db %01010100
	.db %01010111
	.db %01010111
	.db %01010011
	.db %01000111
	.db %00000000
	.db %00000000
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0
	.db %00000001
	.db 0
	;section 2
	;mask
	.db %00000001
	.db %01111100
	.db %01111101
	.db %11111101
	.db %11111101
	.db %11111100
	.db %11101111
	.db %00110101
	.db %11101100
	.db %00101001
	.db %00000000
	.db %00000000
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0
	.db %10111010
	.db 0
	;section 3
	;mask
	.db %11111111
	.db %01111111
	.db %00011111
	.db %01000111
	.db %01010001
	.db %11010101
	.db %11010101
	.db %10010101
	.db %11010101
	.db %10010101
	.db %00000001
	.db %00000001
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
Bullet:
	;2x2
	;mask
	.db %00111111
	.db %00111111
	;sprite
	.db %00000000
	.db %00000000
Missile_Left:
	;Fired from evil blimps
	;4x3
	;mask
	.db %11011111
	.db %00001111
	.db %11011111
	;sprite
	.db 0,0,0
Missile_Right:
	;Fired from evil blimps
	;4x3
	;mask
	.db %10111111
	.db %00001111
	.db %10111111
	;sprite
	.db 0,0,0
People_Sprites:
	.dw RunningMan_Left1
	.dw RunningMan_Left2
	.dw RunningMan_Right1
	.dw RunningMan_Right2
	.dw WavingMan_1
	.dw WavingMan_2
RunningMan_Left1:
	.db %11111011
	.db %11111001
	.db %11110001
	.db %11111011
	.db %11110101
	.db %11111010
	.db 0,0,0,0,0,0
RunningMan_Left2:
	.db %11111011
	.db %11111001
	.db %11101010
	.db %11111011
	.db %11110100
	.db %11110111
	.db 0,0,0,0,0,0
RunningMan_Right1:
	;5x6
	;mask
	.db %11011111
	.db %10011111
	.db %10001111
	.db %11011111
	.db %10101111
	.db %01011111
	;sprite
	.db 0,0,0,0,0,0
RunningMan_Right2:
	;5x6
	;mask
	.db %11011111
	.db %10011111
	.db %01010111
	.db %11011111
	.db %00101111
	.db %11101111
	;sprite
	.db 0,0,0,0,0,0
WavingMan_1:
	;5x6
	;mask
	.db %11011111
	.db %10011111
	.db %01001111
	.db %11011111
	.db %10101111
	.db %10101111
	;sprite
	.db 0,0,0,0,0,0
WavingMan_2:
	;5x6
	;mask
	.db %11011111
	.db %10010111
	.db %01001111
	.db %11011111
	.db %10101111
	.db %10101111
	;sprite
	.db 0,0,0,0,0,0
Tank:
	;8x5
	.db %11100111
	.db %11100111
	.db %10000001
	.db %00000000
	.db %10000001

	.db %00011000
	.db %00011000
	.db %01111110
	.db %11111111
	.db %01111110
Blimp_Left:
	;20x12
	;section 1
	;mask
	.db %11111000
	.db %11100000
	.db %11000000
	.db %10000000
	.db %00000000
	.db %10000000
	.db %11000000
	.db %11100000
	.db %11110000
	.db %11111101
	.db %11111110
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 2
	;mask
	.db %00000111
	.db %00000001
	.db %00000000
	.db %00000100
	.db %00001100
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000001
	.db %00010111
	.db %00001111
	.db %00011111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 3
	;mask
	.db %11011111
	.db %10011111
	.db %00011111
	.db %00111111
	.db %00001111
	.db %00111111
	.db %00011111
	.db %10011111
	.db %11011111
	.db %11111111
	.db %11111111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
Blimp_Right:
	;20x12
	;section 1
	;mask
	.db %10111110
	.db %10011000
	.db %10000000
	.db %11000010
	.db %00000011
	.db %11000000
	.db %10000000
	.db %10010000
	.db %10111000
	.db %11111110
	.db %11111111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 2
	;mask
	.db %00000001
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %00000000
	.db %10001011
	.db %00000111
	.db %10001111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 3
	;mask
	.db %11111111
	.db %01111111
	.db %00111111
	.db %00011111
	.db %00001111
	.db %00011111
	.db %00111111
	.db %01111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
Blimp_Explosion_Sprites:
	.dw Blimp_Exploded_3
	.dw Blimp_Exploded_2
	.dw Blimp_Exploded_1
Blimp_Exploded_1:
	;20x12
	;section 1
	;mask
	.db %11110000
	.db %11010000
	.db %10110000
	.db %00111111
	.db %00100011
	.db %01100000
	.db %11100001
	.db %11100001
	.db %11100001
	.db %11111011
	.db %11110011
	.db %11111011
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 2
	;mask
	.db %00011000
	.db %00001110
	.db %00000011
	.db %00001111
	.db %11111000
	.db %11011000
	.db %10001100
	.db %10000110
	.db %10000011
	.db %11111011
	.db %10111001
	.db %10111011
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 3
	;mask
	.db %11111111
	.db %01111111
	.db %00111111
	.db %11011111
	.db %01111111
	.db %00011111
	.db %00111111
	.db %01111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
Blimp_Exploded_2:
	;20x12
	;section 1
	;mask
	.db %11111111
	.db %11111001
	.db %11110111
	.db %11010010
	.db %10111111
	.db %00100111
	.db %00100100
	.db %01101101
	.db %11111111
	.db %11110001
	.db %11111111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 2
	;mask
	.db %11111111
	.db %00011000
	.db %11001111
	.db %01110011
	.db %00011111
	.db %11111000
	.db %11011011
	.db %10011110
	.db %11110110
	.db %10000111
	.db %11111111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 3
	;mask
	.db %11111111
	.db %11111111
	.db %11111111
	.db %01111111
	.db %10111111
	.db %01011111
	.db %11111111
	.db %01011111
	.db %00111111
	.db %11111111
	.db %11111111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
Blimp_Exploded_3:
	;section 1
	;mask
	.db %11111111
	.db %11111111
	.db %11111101
	.db %11111111
	.db %11010111
	.db %10111111
	.db %01110111
	.db %10101110
	.db %01111101
	.db %11111111
	.db %11110101
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 2
	;mask
	.db %11111111
	.db %11111111
	.db %01011101
	.db %11101111
	.db %01110111
	.db %10111111
	.db %11111101
	.db %11111011
	.db %11011111
	.db %11111110
	.db %11010111
	.db %11111111
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0
	;section 3
	;mask
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %01111111
	.db %10111111
	.db %01011111
	.db %11111111
	.db %01011111
	.db %10111111
	.db %11111111
	.db %11111111
Border:
	;24x19
	;section 1
	;mask
	.db %11111110
	.db %11111110
	.db %11111110
	.db %11111100
	.db %11111101
	.db %11111100
	.db %11111100
	.db %11111110
	.db %11111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %10011111
	.db %10001111
	.db %11010111
	.db %11011011
	.db %11011111
	.db %10000000
	.db %00000000
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	;section 2
	;mask
	.db %10101010
	.db %10101010
	.db %00000000
	.db %00000000
	.db %01010101
	.db %00000000
	.db %00000000
	.db %00000000
	.db %01111101
	.db %00000001
	.db %01010101
	.db %00000001
	.db %00111001
	.db %01010101
	.db %01101101
	.db %01010101
	.db %00111001
	.db %00000000
	.db %00000000
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
	;section 3
	;mask
	.db %11111111
	.db %11111111
	.db %11111111
	.db %01111111
	.db %01111111
	.db %01111111
	.db %01111111
	.db %11111111
	.db %11111111
	.db %11111111
	.db %10000000
	.db %10110110
	.db %10100010
	.db %10110110
	.db %10000000
	.db %11110111
	.db %11110111
	.db %00000001
	.db %00000000
	;sprite
	.db 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
Bullet_Destruct:
	ld (ix),0
	ret
people_data:
	.db 0,0,1 ;bit 7 is set if the man is active
	;bits 5-6 are the house that "owns" the person
	;bit 4 is set if the man has died
	;bit 3 is the animation state (0-1, for swapping pictures)
	;bit 2 is set if currently being airlifted
	;lower 2 bits are the man's current state (00 run left, 01 run right, 10 wave)
	;next byte, countdown for the present state
	;next byte, animation countdown
	.dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	.db 0,0,1 \ .dw ((house0+10)<<6) | $20
	;next house
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	.db %00100000,0,1 \ .dw ((house1+10)<<6) | $20
	;next house
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	.db %01000000,0,1 \ .dw ((house2+10)<<6) | $20
	;next house
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
	.db %01100000,0,1 \ .dw ((house3+10)<<6) | $20
people_data_end:
Person_StateChange:
	;Input - HL is the first byte of the person
	;Output - HL preserved...we give this poor hostage something to do.
	;He can run in the direction of the helicopter (50%)
	;He can run away from the helicopter (25%, must be Stockholm Syndrome)
	;He can wave in an impassioned plea to the user (25%).
	push hl
	;Find the time
	ld b,50
	call ionRandom
	ld b,t_running
	add a,b
	inc hl
	ld (hl),people_update
	inc hl
	ld (hl),a
	inc hl
	ld a,(hl)
	inc hl
	ld h,(hl)
	ld l,a
	ld de,(objects+14)
	or a
	sbc hl,de
	ld c,0
	rl c
	pop hl
	ld b,4
	call ionRandom
	rra
	jr c,Person_StateChange_RunTowards
	rra
	jr c,Person_StateChange_Wave
	;Run away
	ld a,(hl)
	and %11111100
	or c
	xor 1
	ld (hl),a
	ret
Person_StateChange_Wave:
	ld a,(hl)
	and %11111100
	or %00000010
	ld (hl),a
	ret
Person_StateChange_RunTowards:
	ld a,(hl)
	and %11111100
	or c
	ld (hl),a
	ret
chopper_crash:
	ld hl,people
	ld de,5
	ld b,64
chopper_crash_loop:
	ld a,(hl)
	bit 2,a
	jr z,chopper_crash_next
	xor %10010100
	ld (hl),a
chopper_crash_next:
	add hl,de
	djnz chopper_crash_loop
	xor a
	ld (people_chopper),a
	ld b,100
chopper_crash_1:
	ei \ halt
	djnz chopper_crash_1
	jp sortie_start
game_over:
	res indiconly,(iy+indicflags)
	ei
	bcall(_clrlcdfull)
	ld hl,0
	ld (currow),hl
	ld hl,score_text
	bcall(_puts)
	push hl
	ld a,(people_rescued)
	ld l,a
	ld h,0
	bcall(_disphl)
	bcall(_newline)
	pop hl
	bcall(_puts)
	push hl
	ld a,(people_killed)
	ld l,a
	ld h,0
	bcall(_disphl)
	bcall(_newline)
	pop hl
	bcall(_puts)
	push hl
	ld a,64
	ld hl,people_rescued
	sub (hl)
	ld hl,people_killed
	sub (hl)
	ld l,a
	ld h,0
	bcall(_disphl)
	ld hl,5
	ld (currow),hl
	pop hl
	bcall(_puts)
	ld a,(sortie)
	cp 3
	jr z,game_over_1
	inc a
game_over_1:
	ld l,a
	ld h,0
	bcall(_disphl)
game_over_key:
	ei \ halt \ ei \ halt
	bcall(_getcsc)
	or a
	jr z,game_over_key
	ret
score_text:
	.db "Rescued:",0
	.db "Killed: ",0
	.db "Trapped:",0
	.db "Sorties:",0
first_sortie:
	.db "FIRST SORTIE",0
second_sortie:
	.db "SECOND SORTIE",0
third_sortie:
	.db "THIRD SORTIE",0
title_text:
	.db "Kopterlifter",0
	.db 14,"A Poorly Programmed",0
	.db 21,"Production by Michael",0
	.db 28,"Vincent.",0
	.db 36,"2nd - Fire",0
	.db 43,"MODE - Rotate",0
	.db 50,"Arrow keys to move",0
	.db 57,"CLEAR - Pause",0
vputsp:
	ld d,(hl)
	ld e,0
	ld (pencol),de
	inc hl
	bcall(_vputs)
	ret
DefaultSpriteHeight = 8
#include "spr30ax.inc"
.end
